@@ -4,6 +4,7 @@ Most routines work round some numpy oddities in floating point precision and
 casting.  Others work round numpy casting to and from python ints
 """
 
+from numbers import Integral
 from platform import processor, machine
 
 import numpy as np
@@ -13,12 +14,20 @@ class CastingError(Exception):
     pass
 
 
+# Test for VC truncation when casting floats to uint64
+# Christoph Gohlke says this is so for MSVC <= 2010 because VC is using x87
+# instructions; see:
+# https://github.com/scipy/scipy/blob/99bb8411f6391d921cb3f4e56619291e91ddf43b/scipy/ndimage/tests/test_datatypes.py#L51
+_test_val = 2**63 + 2**11 # Should be exactly representable in float64
+TRUNC_UINT64 = np.float64(_test_val).astype(np.uint64) != _test_val
+
+
 def float_to_int(arr, int_type, nan2zero=True, infmax=False):
     """ Convert floating point array `arr` to type `int_type`
 
     * Rounds numbers to nearest integer
     * Clips values to prevent overflows when casting
-    * Converts NaN to 0 (for `nan2zero`==True
+    * Converts NaN to 0 (for `nan2zero` == True)
 
     Casting floats to integers is delicate because the result is undefined
     and platform specific for float values outside the range of `int_type`.
@@ -70,7 +79,7 @@ def float_to_int(arr, int_type, nan2zero=True, infmax=False):
     Hence we threshold at ``shared_min`` and ``shared_max`` to avoid casting to
     values that are undefined.
 
-    See: http://en.wikipedia.org/wiki/C99 . There are links to the C99 standard
+    See: https://en.wikipedia.org/wiki/C99 . There are links to the C99 standard
     from that page.
     """
     arr = np.asarray(arr)
@@ -106,8 +115,8 @@ def shared_range(flt_type, int_type):
     """ Min and max in float type that are >=min, <=max in integer type
 
     This is not as easy as it sounds, because the float type may not be able to
-    exactly represent the max or min integer values, so we have to find the next
-    exactly representable floating point value to do the thresholding.
+    exactly represent the max or min integer values, so we have to find the
+    next exactly representable floating point value to do the thresholding.
 
     Parameters
     ----------
@@ -150,6 +159,8 @@ def shared_range(flt_type, int_type):
     mx = floor_exact(ii.max, flt_type)
     if mx == np.inf:
         mx = fi.max
+    elif TRUNC_UINT64 and int_type == np.uint64:
+        mx = min(mx, flt_type(2**63))
     _SHARED_RANGES[key] = (mn, mx)
     return mn, mx
 
@@ -198,12 +209,13 @@ def type_info(np_type):
 
     Raises
     ------
-    FloatingError : for floating point types we don't recognize
+    FloatingError
+        for floating point types we don't recognize
 
     Notes
     -----
     You might be thinking that ``np.finfo`` does this job, and it does, except
-    for PPC long doubles (http://projects.scipy.org/numpy/ticket/2077) and
+    for PPC long doubles (https://github.com/numpy/numpy/issues/2669) and
     float96 on Windows compiled with Mingw. This routine protects against such
     errors in ``np.finfo`` by only accepting values that we know are likely to
     be correct.
@@ -273,7 +285,7 @@ def type_info(np_type):
         # seems to break here too, so we need to use np.longdouble and
         # complexify
         two = np.longdouble(2)
-        # See: http://matthew-brett.github.com/pydagogue/floating_point.html
+        # See: https://matthew-brett.github.io/pydagogue/floating_point.html
         max_val = (two ** 113 - 1) / (two ** 112) * two ** 16383
         if np_type is np.longcomplex:
             max_val += 0j
@@ -428,6 +440,11 @@ def int_to_float(val, flt_type):
     """
     if not flt_type is np.longdouble:
         return flt_type(val)
+    # The following works around a nasty numpy 1.4.1 bug such that:
+    # >>> int(np.uint32(2**32-1)
+    # -1
+    if not isinstance(val, Integral):
+        val = int(str(val))
     faval = np.longdouble(0)
     while val != 0:
         f64 = np.float64(val)
@@ -584,7 +601,7 @@ def int_abs(arr):
 def floor_log2(x):
     """ floor of log2 of abs(`x`)
 
-    Embarrassingly, from http://en.wikipedia.org/wiki/Binary_logarithm
+    Embarrassingly, from https://en.wikipedia.org/wiki/Binary_logarithm
 
     Parameters
     ----------
@@ -629,21 +646,55 @@ def best_float():
     case we return float64 on the basis it's the fastest and smallest at the
     highest precision.
 
+    SPARC float128 also proved so slow that we prefer float64.
+
     Returns
     -------
     best_type : numpy type
         floating point type with highest precision
+
+    Notes
+    -----
+    Needs to run without error for module import, because it is called in
+    ``ok_floats`` below, and therefore in setting module global ``OK_FLOATS``.
     """
-    if (type_info(np.longdouble)['nmant'] > type_info(np.float64)['nmant'] and
+    try:
+        long_info = type_info(np.longdouble)
+    except FloatingError:
+        return np.float64
+    if (long_info['nmant'] > type_info(np.float64)['nmant'] and
         machine() != 'sparc64'): # sparc has crazy-slow float128
         return np.longdouble
     return np.float64
 
 
+def longdouble_lte_float64():
+    """ Return True if longdouble appears to have the same precision as float64
+    """
+    return np.longdouble(2**53) == np.longdouble(2**53) + 1
+
+
+# Record longdouble precision at import because it can change on Windows
+_LD_LTE_FLOAT64 = longdouble_lte_float64()
+
+
+def longdouble_precision_improved():
+    """ True if longdouble precision increased since initial import
+
+    This can happen on Windows compiled with MSVC.  It may be because libraries
+    compiled with mingw (longdouble is Intel80) get linked to numpy compiled
+    with MSVC (longdouble is Float64)
+    """
+    return not longdouble_lte_float64() and _LD_LTE_FLOAT64
+
+
 def have_binary128():
     """ True if we have a binary128 IEEE longdouble
     """
-    ti = type_info(np.longdouble)
+    try:
+        ti = type_info(np.longdouble)
+    except FloatingError:
+        return False
     return (ti['nmant'], ti['maxexp']) == (112, 16384)
 
 
@@ -652,15 +703,17 @@ def ok_floats():
 
     Remove longdouble if it has no higher precision than float64
     """
-    floats = sorted(np.sctypes['float'], key=lambda f : type_info(f)['nmant'])
+    # copy float list so we don't change the numpy global
+    floats = np.sctypes['float'][:]
     if best_float() != np.longdouble and np.longdouble in floats:
         floats.remove(np.longdouble)
-    return floats
+    return sorted(floats, key=lambda f : type_info(f)['nmant'])
 
 
 OK_FLOATS = ok_floats()
 
 
+
 def able_int_type(values):
     """ Find the smallest integer numpy type to contain sequence `values`
 
