@@ -6,17 +6,56 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
+""" Read ECAT format images
+
+An ECAT format image consists of:
+
+* a *main header*;
+* at least one *matrix list* (mlist);
+
+ECAT thinks of memory locations in terms of *blocks*.  One block is 512
+bytes.  Thus block 1 starts at 0 bytes, block 2 at 512 bytes, and so on.
+
+The matrix list is an array with one row per frame in the data.
+
+Columns in the matrix list are:
+
+* 0 - Matrix identifier (frame number)
+* 1 - matrix data start block number (subheader followed by image data)
+* 2 - Last block number of matrix (image) data
+* 3 - Matrix status:
+    * 1 - exists - rw
+    * 2 - exists - ro
+    * 3 - matrix deleted
+
+There is one sub-header for each image frame (or matrix in the terminology
+above).  A sub-header can also be called an *image header*.  The sub-header is
+one block (512 bytes), and the frame (image) data follows.
+
+There is very little documentation of the ECAT format, and many of the comments
+in this code come from a combination of trial and error and wild speculation.
+
+XMedcon can read and write ECAT 6 format, and read ECAT 7 format: see
+http://xmedcon.sourceforge.net and the ECAT files in the source of XMedCon,
+currently ``libs/tpc/*ecat*`` and ``source/m-ecat*``.  Unfortunately XMedCon is
+GPL and some of the header files are adapted from CTI files (called CTI code
+below).  It's not clear what the licenses are for these files.
+"""
+
 import warnings
+from numbers import Integral
 
 import numpy as np
 
 from .volumeutils import (native_code, swapped_code, make_dt_codes,
                            array_from_file)
-from .spatialimages import SpatialImage, ImageDataError
+from .spatialimages import SpatialImage
 from .arraywriters import make_array_writer
+from .wrapstruct import WrapStruct
+from .fileslice import canonical_slicers, predict_shape, slice2outax
 
+BLOCK_SIZE = 512
 
-MAINHDRSZ = 502
 main_header_dtd = [
     ('magic_number', '14S'),
     ('original_filename', '32S'),
@@ -147,14 +186,19 @@ subheader_dtd = [
 subhdr_dtype = np.dtype(subheader_dtd)
 
 # Ecat Data Types
+# See:
+# http://www.turkupetcentre.net/software/libdoc/libtpcimgio/ecat7_8h_source.html#l00060
+# and:
+# http://www.turkupetcentre.net/software/libdoc/libtpcimgio/ecat7r_8c_source.html#l00717
 _dtdefs = ( # code, name, equivalent dtype
-    (1, 'ECAT7_BYTE', np.uint8),
-    (2, 'ECAT7_VAXI2', np.int16),
-    (3, 'ECAT7_VAXI4', np.float32),
-    (4, 'ECAT7_VAXR4', np.float32),
-    (5, 'ECAT7_IEEER4', np.float32),
-    (6, 'ECAT7_SUNI2', np.uint16),
-    (7, 'ECAT7_SUNI4', np.int32))
+           (1, 'ECAT7_BYTE', np.uint8),
+           # Byte signed? https://github.com/nipy/nibabel/pull/302/files#r28275780
+           (2, 'ECAT7_VAXI2', np.int16),
+           (3, 'ECAT7_VAXI4', np.int32),
+           (4, 'ECAT7_VAXR4', np.float32),
+           (5, 'ECAT7_IEEER4', np.float32),
+           (6, 'ECAT7_SUNI2', np.int16),
+           (7, 'ECAT7_SUNI4', np.int32))
 data_type_codes = make_dt_codes(_dtdefs)
 
 
@@ -175,8 +219,9 @@ ft_defs = ( # code, name
     (12, 'ECAT7_3DSCAN8'),
     (13, 'ECAT7_3DNORM'),
     (14, 'ECAT7_3DSCANFIT'))
+file_type_codes = dict(ft_defs)
 
-patient_orient_defs = ( #code, description
+patient_orient_defs = ( # code, description
     (0, 'ECAT7_Feet_First_Prone'),
     (1, 'ECAT7_Head_First_Prone'),
     (2, 'ECAT7_Feet_First_Supine'),
@@ -186,81 +231,53 @@ patient_orient_defs = ( #code, description
     (6, 'ECAT7_Feet_First_Decubitus_Left'),
     (7, 'ECAT7_Head_First_Decubitus_Left'),
     (8, 'ECAT7_Unknown_Orientation'))
+patient_orient_codes = dict(patient_orient_defs)
 
 #Indexes from the patient_orient_defs structure defined above for the
 #neurological and radiological viewing conventions
 patient_orient_radiological = [0, 2, 4, 6]
 patient_orient_neurological = [1, 3, 5, 7]
 
-class EcatHeader(object):
+class EcatHeader(WrapStruct):
     """Class for basic Ecat PET header
+
     Sub-parts of standard Ecat File
-       main header
-       matrix list
-           which lists the information for each
-           frame collected (can have 1 to many frames)
-       subheaders specific to each frame
-           with possibly-variable sized data blocks
 
-    This just reads the main Ecat Header,
-    it does not load the data
-    or read the mlist or any sub headers
+    * main header
+    * matrix list
+      which lists the information for each frame collected (can have 1 to many
+      frames)
+    * subheaders specific to each frame with possibly-variable sized data blocks
 
+    This just reads the main Ecat Header, it does not load the data or read the
+    mlist or any sub headers
     """
-
-    _dtype = hdr_dtype
-    _ft_defs = ft_defs
-    _patient_orient_defs = patient_orient_defs
+    template_dtype = hdr_dtype
+    _ft_codes = file_type_codes
+    _patient_orient_codes = patient_orient_codes
 
     def __init__(self,
-                 fileobj=None,
-                 endianness=None):
-        """Initialize Ecat header from file object
+                 binaryblock=None,
+                 endianness=None,
+                 check=True):
+        """Initialize Ecat header from bytes object
 
         Parameters
         ----------
-        fileobj : {None, string} optional
-            binary block to set into header, By default, None
-            in which case we insert default empty header block
+        binaryblock : {None, bytes} optional
+            binary block to set into header, By default, None in which case we
+            insert default empty header block
         endianness : {None, '<', '>', other endian code}, optional
             endian code of binary block, If None, guess endianness
             from the data
+        check : {True, False}, optional
+            Whether to check and fix header for errors.  No checks currently
+            implemented, so value has no effect.
         """
-        if fileobj is None:
-            self._header_data = self._empty_headerdata(endianness)
-            return
+        super(EcatHeader, self).__init__(binaryblock, endianness, check)
 
-        hdr = np.ndarray(shape=(),
-                         dtype=self._dtype,
-                         buffer=fileobj)
-        if endianness is None:
-            endianness = self._guess_endian(hdr)
-
-        if endianness != native_code:
-            dt = self._dtype.newbyteorder(endianness)
-            hdr = np.ndarray(shape=(),
-                             dtype=dt,
-                             buffer=fileobj)
-        self._header_data = hdr.copy()
-
-        return
-
-    def get_header(self):
-        """returns header """
-        return self
-
-    @property
-    def binaryblock(self):
-        return self._header_data.tostring()
-
-    @property
-    def endianness(self):
-        if self._header_data.dtype.isnative:
-            return native_code
-        return swapped_code
-
-
-    def _guess_endian(self, hdr):
+    @classmethod
+    def guessed_endian(klass, hdr):
         """Guess endian from MAGIC NUMBER value of header data
         """
         if not hdr['sw_version'] == 74:
@@ -269,39 +286,10 @@ class EcatHeader(object):
             return native_code
 
     @classmethod
-    def from_fileobj(klass, fileobj, endianness=None):
-        """Return /read header with given or guessed endian code
-
-        Parameters
-        ----------
-        fileobj : file-like object
-            Needs to implement ``read`` method
-        endianness : None or endian code, optional
-            Code specifying endianness of data to be read
-
-        Returns
-        -------
-        hdr : EcatHeader object
-            EcatHeader object initialized from data in file object
-
-        Examples
-        --------
-
-
-        """
-        raw_str = fileobj.read(klass._dtype.itemsize)
-        return klass(raw_str, endianness)
-
-    def write_to(self, fileobj):
-        fileobj.write(self.binaryblock)
-
-    def _empty_headerdata(self,endianness=None):
-        """Return header data for empty header with given endianness"""
-        #hdr_data = super(EcatHeader, self)._empty_headerdata(endianness)
-        dt = self._dtype
-        if not endianness is None:
-            dt = dt.newbyteorder(endianness)
-        hdr_data = np.zeros((), dtype=dt)
+    def default_structarr(klass, endianness=None):
+        ''' Return header data for empty header with given endianness
+        '''
+        hdr_data = super(EcatHeader, klass).default_structarr(endianness)
         hdr_data['magic_number'] = 'MATRIX72'
         hdr_data['sw_version'] = 74
         hdr_data['num_frames']= 0
@@ -309,232 +297,215 @@ class EcatHeader(object):
         hdr_data['ecat_calibration_factor'] = 1.0 # scale factor
         return hdr_data
 
-
     def get_data_dtype(self):
         """ Get numpy dtype for data from header"""
         raise NotImplementedError("dtype is only valid from subheaders")
 
-
-    def copy(self):
-        return self.__class__(
-            self.binaryblock,
-            self.endianness)
-
-
-    def __eq__(self, other):
-        """ checks for equality between two headers"""
-        self_end = self.endianness
-        self_bb = self.binaryblock
-        if self_end == other.endianness:
-            return self_bb == other.binaryblock
-        other_bb = other._header_data.byteswap().tostring()
-        return self_bb == other_bb
-
-    def __ne__(self, other):
-        ''' equality between two headers defined by ``header_data``
-
-        For examples, see ``__eq__`` method docstring
-        '''
-        return not self == other
-
-    def __getitem__(self, item):
-        ''' Return values from header data
-
-        Examples
-        --------
-        >>> hdr = EcatHeader()
-        >>> hdr['magic_number'] #23dt next : bytes
-        'MATRIX72'
-        '''
-        return self._header_data[item].item()
-
-    def __setitem__(self, item, value):
-        ''' Set values in header data
-
-        Examples
-        --------
-        >>> hdr = EcatHeader()
-        >>> hdr['num_frames'] = 2
-        >>> hdr['num_frames']
-        2
-        '''
-        self._header_data[item] = value
-
     def get_patient_orient(self):
         """ gets orientation of patient based on code stored
         in header, not always reliable"""
-        orient_code = dict(self._patient_orient_defs)
-        code = self._header_data['patient_orientation'].item()
-        if not orient_code.has_key(code):
-            raise KeyError('Ecat Orientation CODE %d not recognized'%code)
-        return orient_code[code]
+        code = self._structarr['patient_orientation'].item()
+        if not code in self._patient_orient_codes:
+            raise KeyError('Ecat Orientation CODE %d not recognized' % code)
+        return self._patient_orient_codes[code]
 
     def get_filetype(self):
-        """ gets type of ECAT Matrix File from
-        code stored in header"""
-        ft_codes = dict(self._ft_defs)
-        code = self._header_data['file_type'].item()
-        if not ft_codes.has_key(code):
-            raise KeyError('Ecat Filetype CODE %d not recognized'%code)
-        return ft_codes[code]
+        """ Type of ECAT Matrix File from code stored in header"""
+        code = self._structarr['file_type'].item()
+        if not code in self._ft_codes:
+            raise KeyError('Ecat Filetype CODE %d not recognized' % code)
+        return self._ft_codes[code]
 
-    def __iter__(self):
-        return iter(self.keys())
-
-    def keys(self):
-        ''' Return keys from header data'''
-        return list(self._dtype.names)
-
-    def values(self):
-        ''' Return values from header data'''
-        data = self._header_data
-        return [data[key] for key in self._dtype.names]
-
-    def items(self):
-        ''' Return items from header data'''
-        return zip(self.keys(), self.values())
-
-class EcatMlist(object):
-
-    def __init__(self,fileobj, hdr):
-        """ gets list of frames and subheaders in pet file
-
-        Parameters
-        -----------
-        fileobj : ECAT file <filename>.v  fileholder or file object
-                  with read, seek methods
-
-        Returns
-        -------
-        mlist : numpy recarray  nframes X 4 columns
-        1 - Matrix identifier.
-        2 - subheader record number
-        3 - Last record number of matrix data block.
-        4 - Matrix status:
-            1 - exists - rw
-            2 - exists - ro
-            3 - matrix deleted
-        """
-        self.hdr = hdr
-        self._mlist = self.get_mlist(fileobj)
-
-    def get_mlist(self, fileobj):
-        fileobj.seek(512)
-        dat=fileobj.read(128*32)
-
-        dt = np.dtype([('matlist',np.int32)])
-        if not self.hdr.endianness is native_code:
-            dt = dt.newbyteorder(self.hdr.endianness)
-        nframes = self.hdr['num_frames']
-        mlist = np.zeros((nframes,4), dtype='uint32')
-        record_count = 0
-        done = False
-
-        while not done: #mats['matlist'][0,1] == 2:
-
-            mats = np.recarray(shape=(32,4), dtype=dt,  buf=dat)
-            if not (mats['matlist'][0,0] +  mats['matlist'][0,3]) == 31:
-                mlist = []
-                return mlist
-
-            nrecords = mats['matlist'][0,3]
-            mlist[record_count:nrecords+record_count,:] = mats['matlist'][1:nrecords+1,:]
-            record_count+= nrecords
-            if mats['matlist'][0,1] == 2 or mats['matlist'][0,1] == 0:
-                done = True
-            else:
-                # Find next subheader
-                tmp = int(mats['matlist'][0,1]-1)#cast to int
-                fileobj.seek(0)
-                fileobj.seek(tmp*512)
-                dat = fileobj.read(128*32)
-
-        return mlist
-
-    def get_frame_order(self):
-        """Returns the order of the frames stored in the file
-        Sometimes Frames are not stored in the file in
-        chronological order, this can be used to extract frames
-        in correct order
-
-        Returns
-        -------
-        id_dict: dict mapping frame number -> [mlist_row, mlist_id]
-
-        (where mlist id is value in the first column of the mlist matrix )
-
-        Examples
-        --------
-        >>> import os
-        >>> import nibabel as nib
-        >>> nibabel_dir = os.path.dirname(nib.__file__)
-        >>> from nibabel import ecat
-        >>> ecat_file = os.path.join(nibabel_dir,'tests','data','tinypet.v')
-        >>> img = ecat.load(ecat_file)
-        >>> mlist = img.get_mlist()
-        >>> mlist.get_frame_order()
-        {0: [0, 16842758]}
-        """
-        mlist  = self._mlist
-        ids = mlist[:, 0].copy()
-        n_valid = np.sum(ids > 0)
-        ids[ids <=0] = ids.max() + 1 # put invalid frames at end after sort
-        valid_order = np.argsort(ids)
-        if not all(valid_order == sorted(valid_order)):
-            #raise UserWarning if Frames stored out of order
-            warnings.warn_explicit('Frames stored out of order;'\
-                          'true order = %s\n'\
-                          'frames will be accessed in order '\
-                          'STORED, NOT true order'%(valid_order),
-                          UserWarning,'ecat', 0)
-        id_dict = {}
-        for i in range(n_valid):
-            id_dict[i] = [valid_order[i], ids[valid_order[i]]]
-
-        return id_dict
-
-    def get_series_framenumbers(self):
-        """ Returns framenumber of data as it was collected,
-        as part of a series; not just the order of how it was
-        stored in this or across other files
-
-        For example, if the data is split between multiple files
-        this should give you the true location of this frame as
-        collected in the series
-        (Frames are numbered starting at ONE (1) not Zero)
-
-        Returns
-        -------
-        frame_dict: dict mapping order_stored -> frame in series
-               where frame in series counts from 1; [1,2,3,4...]
-
-        Examples
-        --------
-        >>> import os
-        >>> import nibabel as nib
-        >>> nibabel_dir = os.path.dirname(nib.__file__)
-        >>> from nibabel import ecat
-        >>> ecat_file = os.path.join(nibabel_dir,'tests','data','tinypet.v')
-        >>> img = ecat.load(ecat_file)
-        >>> mlist = img.get_mlist()
-        >>> mlist.get_series_framenumbers()
-        {0: 1}
+    @classmethod
+    def _get_checks(klass):
+        ''' Return sequence of check functions for this class '''
+        return ()
 
 
+def read_mlist(fileobj, endianness):
+    """ read (nframes, 4) matrix list array from `fileobj`
+
+    Parameters
+    ----------
+    fileobj : file-like
+        an open file-like object implementing ``seek`` and ``read``
+
+    Returns
+    -------
+    mlist : (nframes, 4) ndarray
+        matrix list is an array with ``nframes`` rows and columns:
+
+        * 0 - Matrix identifier (frame number)
+        * 1 - matrix data start block number (subheader followed by image data)
+        * 2 - Last block number of matrix (image) data
+        * 3 - Matrix status:
+            * 1 - exists - rw
+            * 2 - exists - ro
+            * 3 - matrix deleted
+
+    Notes
+    -----
+    A block is 512 bytes.
+
+    ``block_no`` in the code below is 1-based.  block 1 is the main header,
+    and the mlist blocks start at block number 2.
+
+    The 512 bytes in an mlist block contain 32 rows of the int32 (nframes,
+    4) mlist matrix.
+
+    The first row of these 32 looks like a special row.  The 4 values appear
+    to be (respectively):
+
+    * not sure - maybe negative number of mlist rows (out of 31) that are
+      blank and not used in this block.  Called `nfree` but unused in CTI
+      code;
+    * block_no - of next set of mlist entries or 2 if no more entries. We also
+      allow 1 or 0 to signal no more entries;
+    * <no idea>.  Called `prvblk` in CTI code, so maybe previous block no;
+    * n_rows - number of mlist rows in this block (between ?0 and 31) (called
+      `nused` in CTI code).
+    """
+    dt = np.dtype(np.int32)
+    if not endianness is native_code:
+        dt = dt.newbyteorder(endianness)
+    mlists = []
+    mlist_index = 0
+    mlist_block_no = 2  # 1-based indexing, block with first mlist
+    while True:
+        # Read block containing mlist entries
+        fileobj.seek((mlist_block_no - 1) * BLOCK_SIZE) # fix 1-based indexing
+        dat = fileobj.read(BLOCK_SIZE)
+        rows = np.ndarray(shape=(32, 4), dtype=dt, buffer=dat)
+        # First row special, points to next mlist entries if present
+        n_unused, mlist_block_no, _, n_rows = rows[0]
+        if not (n_unused + n_rows) == 31: # Some error condition here?
+            mlist = []
+            return mlist
+        # Use all but first housekeeping row
+        mlists.append(rows[1:n_rows+1])
+        mlist_index += n_rows
+        if mlist_block_no <= 2: # should block_no in (1, 2) be an error?
+            break
+    return np.row_stack(mlists)
+
+
+def get_frame_order(mlist):
+    """Returns the order of the frames stored in the file
+    Sometimes Frames are not stored in the file in
+    chronological order, this can be used to extract frames
+    in correct order
+
+    Returns
+    -------
+    id_dict: dict mapping frame number -> [mlist_row, mlist_id]
+
+    (where mlist id is value in the first column of the mlist matrix )
+
+    Examples
+    --------
+    >>> import os
+    >>> import nibabel as nib
+    >>> nibabel_dir = os.path.dirname(nib.__file__)
+    >>> from nibabel import ecat
+    >>> ecat_file = os.path.join(nibabel_dir,'tests','data','tinypet.v')
+    >>> img = ecat.load(ecat_file)
+    >>> mlist = img.get_mlist()
+    >>> get_frame_order(mlist)
+    {0: [0, 16842758]}
+    """
+    ids = mlist[:, 0].copy()
+    n_valid = np.sum(ids > 0)
+    ids[ids <=0] = ids.max() + 1 # put invalid frames at end after sort
+    valid_order = np.argsort(ids)
+    if not all(valid_order == sorted(valid_order)):
+        #raise UserWarning if Frames stored out of order
+        warnings.warn_explicit('Frames stored out of order;'\
+                        'true order = %s\n'\
+                        'frames will be accessed in order '\
+                        'STORED, NOT true order'%(valid_order),
+                        UserWarning,'ecat', 0)
+    id_dict = {}
+    for i in range(n_valid):
+        id_dict[i] = [valid_order[i], ids[valid_order[i]]]
+    return id_dict
+
+
+def get_series_framenumbers(mlist):
+    """ Returns framenumber of data as it was collected,
+    as part of a series; not just the order of how it was
+    stored in this or across other files
+
+    For example, if the data is split between multiple files
+    this should give you the true location of this frame as
+    collected in the series
+    (Frames are numbered starting at ONE (1) not Zero)
+
+    Returns
+    -------
+    frame_dict: dict mapping order_stored -> frame in series
+            where frame in series counts from 1; [1,2,3,4...]
+
+    Examples
+    --------
+    >>> import os
+    >>> import nibabel as nib
+    >>> nibabel_dir = os.path.dirname(nib.__file__)
+    >>> from nibabel import ecat
+    >>> ecat_file = os.path.join(nibabel_dir,'tests','data','tinypet.v')
+    >>> img = ecat.load(ecat_file)
+    >>> mlist = img.get_mlist()
+    >>> get_series_framenumbers(mlist)
+    {0: 1}
+    """
+    nframes = len(mlist)
+    frames_order = get_frame_order(mlist)
+    mlist_nframes = len(frames_order)
+    trueframenumbers = np.arange(nframes - mlist_nframes, nframes)
+    frame_dict = {}
+    try:
+        for frame_stored, (true_order, _) in frames_order.items():
+            #frame as stored in file -> true number in series
+            frame_dict[frame_stored] = trueframenumbers[true_order]+1
+        return frame_dict
+    except:
+        raise IOError('Error in header or mlist order unknown')
+
+
+def read_subheaders(fileobj, mlist, endianness):
+    """ Retrieve all subheaders and return list of subheader recarrays
+
+    Parameters
+    ----------
+    fileobj : file-like
+        implementing ``read`` and ``seek``
+    mlist : (nframes, 4) ndarray
+        Columns are:
+        * 0 - Matrix identifier.
+        * 1 - subheader block number
+        * 2 - Last block number of matrix data block.
+        * 3 - Matrix status:
+    endianness : {'<', '>'}
+        little / big endian code
+
+    Returns
+    -------
+    subheaders : list
+        List of subheader structured arrays
+    """
+    subheaders = []
+    dt = subhdr_dtype
+    if not endianness is native_code:
+        dt = dt.newbyteorder(endianness)
+    for mat_id, sh_blkno, sh_last_blkno, mat_stat in mlist:
+        if sh_blkno == 0:
+            break
+        offset = (sh_blkno - 1) * BLOCK_SIZE
+        fileobj.seek(offset)
+        tmpdat = fileobj.read(BLOCK_SIZE)
+        sh = np.ndarray(shape=(), dtype=dt, buffer=tmpdat)
+        subheaders.append(sh)
+    return subheaders
 
-        """
-        frames_order = self.get_frame_order()
-        nframes = self.hdr['num_frames']
-        mlist_nframes = len(frames_order)
-        trueframenumbers = np.arange(nframes - mlist_nframes, nframes)
-        frame_dict = {}
-        try:
-            for frame_stored, (true_order, _) in frames_order.items():
-                #frame as stored in file -> true number in series
-                frame_dict[frame_stored] = trueframenumbers[true_order]+1
-            return frame_dict
-        except:
-            raise IOError('Error in header or mlist order unknown')
 
 class EcatSubHeader(object):
 
@@ -548,49 +519,17 @@ class EcatSubHeader(object):
         Parameters
         -----------
         hdr : EcatHeader
-
-        mlist : EcatMlist
-
+            ECAT main header
+        mlist : array shape (N, 4)
+            Matrix list
         fileobj : ECAT file <filename>.v  fileholder or file object
                   with read, seek methods
-
-
         """
         self._header = hdr
         self.endianness = hdr.endianness
         self._mlist = mlist
         self.fileobj = fileobj
-        self.subheaders = self._get_subheaders()
-
-    def _get_subheaders(self):
-        """retreive all subheaders and return list of subheader recarrays
-        """
-        subheaders = []
-        header = self._header
-        endianness = self.endianness
-        dt = self._subhdrdtype
-        if not self.endianness is native_code:
-            dt = self._subhdrdtype.newbyteorder(self.endianness)
-        if self._header['num_frames'] > 1:
-            for item in self._mlist._mlist:
-                if item[1] == 0:
-                    break
-                self.fileobj.seek(0)
-                offset = (int(item[1])-1)*512
-                self.fileobj.seek(offset)
-                tmpdat = self.fileobj.read(512)
-                sh = (np.recarray(shape=(), dtype=dt,
-                                  buf=tmpdat))
-                subheaders.append(sh.copy())
-        else:
-            self.fileobj.seek(0)
-            offset = (int(self._mlist._mlist[0][1])-1)*512
-            self.fileobj.seek(offset)
-            tmpdat = self.fileobj.read(512)
-            sh = (np.recarray(shape=(), dtype=dt,
-                              buf=tmpdat))
-            subheaders.append(sh)
-        return subheaders
+        self.subheaders = read_subheaders(fileobj, mlist, hdr.endianness)
 
     def get_shape(self, frame=0):
         """ returns shape of given frame"""
@@ -602,11 +541,9 @@ class EcatSubHeader(object):
 
     def get_nframes(self):
         """returns number of frames"""
-        mlist = self._mlist
-        framed = mlist.get_frame_order()
+        framed = get_frame_order(self._mlist)
         return len(framed)
 
-
     def _check_affines(self):
         """checks if all affines are equal across frames"""
         nframes = self.get_nframes()
@@ -651,9 +588,7 @@ class EcatSubHeader(object):
         return self._data_type_codes.dtype[dtcode]
 
     def _get_frame_offset(self, frame=0):
-        mlist = self._mlist._mlist
-        offset = (mlist[frame][1]) * 512
-        return int(offset)
+        return int(self._mlist[frame][1] * BLOCK_SIZE)
 
     def _get_oriented_data(self, raw_data, orientation=None):
         '''
@@ -715,52 +650,92 @@ class EcatSubHeader(object):
         header = self._header
         subhdr = self.subheaders[frame]
         raw_data = self.raw_data_from_fileobj(frame, orientation)
-        data = raw_data * header['ecat_calibration_factor']
-        data = data * subhdr['scale_factor']
+        # Scale factors have to be set to scalars to force scalar upcasting
+        data = raw_data * np.asscalar(header['ecat_calibration_factor'])
+        data = data * np.asscalar(subhdr['scale_factor'])
         return data
 
 
+class EcatImageArrayProxy(object):
+    ''' Ecat implemention of array proxy protocol
+
+    The array proxy allows us to freeze the passed fileobj and
+    header such that it returns the expected data array.
+    '''
+    def __init__(self, subheader):
+        self._subheader = subheader
+        self._data = None
+        x, y, z = subheader.get_shape()
+        nframes = subheader.get_nframes()
+        self._shape = (x, y, z, nframes)
+
+    @property
+    def shape(self):
+        return self._shape
+
+    @property
+    def is_proxy(self):
+        return True
+
+    def __array__(self):
+        ''' Read of data from file
+
+        This reads ALL FRAMES into one array, can be memory expensive.
+
+        If you want to read only some slices, use the slicing syntax
+        (``__getitem__``) below, or ``subheader.data_from_fileobj(frame)``
+        '''
+        data = np.empty(self.shape)
+        frame_mapping = get_frame_order(self._subheader._mlist)
+        for i in sorted(frame_mapping):
+            data[:,:,:,i] = self._subheader.data_from_fileobj(frame_mapping[i][0])
+        return data
+
+    def __getitem__(self, sliceobj):
+        """ Return slice `sliceobj` from ECAT data, optimizing if possible
+        """
+        sliceobj = canonical_slicers(sliceobj, self.shape)
+        # Indices into sliceobj referring to image axes
+        ax_inds = [i for i, obj in enumerate(sliceobj) if not obj is None]
+        assert len(ax_inds) == len(self.shape)
+        frame_mapping = get_frame_order(self._subheader._mlist)
+        # Analyze index for 4th axis
+        slice3 = sliceobj[ax_inds[3]]
+        # We will load volume by volume.  Make slicer into volume by dropping
+        # index over the volume axis
+        in_slicer = sliceobj[:ax_inds[3]] + sliceobj[ax_inds[3]+1:]
+        # int index for 4th axis, load one slice
+        if isinstance(slice3, Integral):
+            data = self._subheader.data_from_fileobj(frame_mapping[slice3][0])
+            return data[in_slicer]
+        # slice axis for 4th axis, we will iterate over slices
+        out_shape = predict_shape(sliceobj, self.shape)
+        out_data = np.empty(out_shape)
+        # Slice into output data with out_slicer
+        out_slicer = [slice(None)] * len(out_shape)
+        # Work out axis corresponding to volume in output
+        in2out_ind = slice2outax(len(self.shape), sliceobj)[3]
+        # Iterate over specified 4th axis indices
+        for i in list(range(self.shape[3]))[slice3]:
+            data = self._subheader.data_from_fileobj(
+                frame_mapping[i][0])
+            out_slicer[in2out_ind] = i
+            out_data[tuple(out_slicer)] = data[in_slicer]
+        return out_data
 
 
 class EcatImage(SpatialImage):
-    """This class returns a list of Ecat images,
-    with one image(hdr/data) per frame
+    """ Class returns a list of Ecat images, with one image(hdr/data) per frame
     """
     _header = EcatHeader
     header_class = _header
     _subheader = EcatSubHeader
-    _mlist = EcatMlist
     files_types = (('image', '.v'), ('header', '.v'))
 
+    ImageArrayProxy = EcatImageArrayProxy
 
-    class ImageArrayProxy(object):
-        ''' Ecat implemention of array proxy protocol
-
-        The array proxy allows us to freeze the passed fileobj and
-        header such that it returns the expected data array.
-        '''
-        def __init__(self, subheader):
-            self._subheader = subheader
-            self._data = None
-            x, y, z = subheader.get_shape()
-            nframes = subheader.get_nframes()
-            self.shape = (x, y, z, nframes)
-
-        def __array__(self):
-            ''' Cached read of data from file
-            This reads ALL FRAMES into one array, can be memory expensive
-            use subheader.data_from_fileobj(frame) for less memory intensive
-            reads
-            '''
-            if self._data is None:
-                self._data = np.empty(self.shape)
-                frame_mapping = self._subheader._mlist.get_frame_order()
-                for i in sorted(frame_mapping):
-                    self._data[:,:,:,i] = self._subheader.data_from_fileobj(frame_mapping[i][0])
-            return self._data
-
-    def __init__(self, data, affine, header,
-                 subheader, mlist ,
+    def __init__(self, dataobj, affine, header,
+                 subheader, mlist,
                  extra = None, file_map = None):
         """ Initialize Image
 
@@ -771,7 +746,7 @@ class EcatImage(SpatialImage):
 
         Parameters
         ----------
-        data : None or array-like
+        dataabj : array-like
             image data
         affine : None or (4,4) array-like
             homogeneous affine giving relationship between voxel coords and
@@ -780,8 +755,8 @@ class EcatImage(SpatialImage):
             meta data for this image format
         subheader : None or subheader instance
             meta data for each sub-image for frame in the image
-        mlist : None or mlist instance
-            meta data with array giving offset and order of data in file
+        mlist : None or array
+            Matrix list array giving offset and order of data in file
         extra : None or mapping, optional
             metadata associated with this image that cannot be
             stored in header or subheader
@@ -805,12 +780,12 @@ class EcatImage(SpatialImage):
         """
         self._subheader = subheader
         self._mlist = mlist
-        self._data = data
+        self._dataobj = dataobj
         if not affine is None:
             # Check that affine is array-like 4,4.  Maybe this is too strict at
             # this abstract level, but so far I think all image formats we know
             # do need 4,4.
-            affine = np.asarray(affine)
+            affine = np.array(affine, dtype=np.float64, copy=True)
             if not affine.shape == (4,4):
                 raise ValueError('Affine should be shape 4,4')
         self._affine = affine
@@ -821,18 +796,10 @@ class EcatImage(SpatialImage):
         if file_map is None:
             file_map = self.__class__.make_file_map()
         self.file_map = file_map
+        self._data_cache = None
 
-    def _set_header(self, header):
-        self._header = header
-
-    def get_data(self):
-        """returns scaled data for all frames in a numpy array
-        returns as a 4D array """
-        if self._data is None:
-            raise ImageDataError('No data in this image')
-        return np.asanyarray(self._data)
-
-    def get_affine(self):
+    @property
+    def affine(self):
         if not self._subheader._check_affines():
             warnings.warn('Affines different across frames, loading affine from FIRST frame',
                           UserWarning )
@@ -864,7 +831,8 @@ class EcatImage(SpatialImage):
         return(x, y, z, nframes)
 
     def get_mlist(self):
-        """ get access to the mlist """
+        """ get access to the mlist
+        """
         return self._mlist
 
     def get_subheaders(self):
@@ -875,7 +843,6 @@ class EcatImage(SpatialImage):
     def from_filespec(klass, filespec):
         return klass.from_filename(filespec)
 
-
     @staticmethod
     def _get_fileholders(file_map):
         """ returns files specific to header and image of the image
@@ -898,21 +865,21 @@ class EcatImage(SpatialImage):
         header = klass._header.from_fileobj(hdr_fid)
         hdr_copy = header.copy()
         ### LOAD MLIST
-        mlist = klass._mlist(hdr_fid, hdr_copy)
+        mlist = np.zeros((header['num_frames'], 4), dtype=np.int32)
+        mlist_data = read_mlist(hdr_fid, hdr_copy.endianness)
+        mlist[:len(mlist_data)] = mlist_data
         ### LOAD SUBHEADERS
-        subheaders = klass._subheader(hdr_copy,
-                                      mlist,
-                                      hdr_fid)
+        subheaders = klass._subheader(hdr_copy, mlist, hdr_fid)
         ### LOAD DATA
         ##  Class level ImageArrayProxy
         data = klass.ImageArrayProxy(subheaders)
-
         ## Get affine
         if not subheaders._check_affines():
-            warnings.warn('Affines different across frames, loading affine from FIRST frame',
-                          UserWarning )
+            warnings.warn('Affines different across frames, loading affine '
+                          'from FIRST frame', UserWarning )
         aff = subheaders.get_frame_affine()
-        img = klass(data, aff, header, subheaders, mlist, extra=None, file_map = file_map)
+        img = klass(data, aff, header, subheaders, mlist,
+                    extra=None, file_map=file_map)
         return img
 
     def _get_empty_dir(self):
@@ -920,7 +887,7 @@ class EcatImage(SpatialImage):
         Get empty directory entry of the form
         [numAvail, nextDir, previousDir, numUsed]
         '''
-        return np.array([31, 2, 0, 0], dtype=np.uint32)
+        return np.array([31, 2, 0, 0], dtype=np.int32)
 
     def _write_data(self, data, stream, pos, dtype=None, endianness=None):
         '''
@@ -956,12 +923,14 @@ class EcatImage(SpatialImage):
         if file_map is None:
             file_map = self.file_map
 
+        # It appears to be necessary to load the data before saving even if the
+        # data itself is not used.
         data = self.get_data()
         hdr = self.get_header()
-        mlist = self.get_mlist()._mlist
+        mlist = self._mlist
         subheaders = self.get_subheaders()
-        dir_pos = 512L
-        entry_pos = dir_pos + 16L #528L
+        dir_pos = 512
+        entry_pos = dir_pos + 16 #528
         current_dir = self._get_empty_dir()
 
         hdr_fh, img_fh = self._get_fileholders(file_map)
@@ -972,7 +941,7 @@ class EcatImage(SpatialImage):
         hdr.write_to(hdrf)
 
         #Write every frames
-        for index in xrange(0, self.get_header()['num_frames']):
+        for index in range(0, self.get_header()['num_frames']):
             #Move to subheader offset
             frame_offset = subheaders._get_frame_offset(index) - 512
             imgf.seek(frame_offset)
@@ -993,10 +962,9 @@ class EcatImage(SpatialImage):
             self._write_data(image, imgf, pos+2, endianness='>')
 
             #Move to dictionnary offset and write dictionnary entry
-            self._write_data(mlist[index], imgf, entry_pos,
-                np.uint32, endianness='>')
+            self._write_data(mlist[index], imgf, entry_pos, endianness='>')
 
-            entry_pos = entry_pos + 16L
+            entry_pos = entry_pos + 16
 
             current_dir[0] = current_dir[0] - 1
             current_dir[3] = current_dir[3] + 1
@@ -1006,17 +974,17 @@ class EcatImage(SpatialImage):
                 #self._write_dir(current_dir, imgf, dir_pos)
                 self._write_data(current_dir, imgf, dir_pos)
                 current_dir = self._get_empty_dir()
-                current_dir[3] = dir_pos / 512L
+                current_dir[3] = dir_pos / 512
                 dir_pos = mlist[index][2] + 1
-                entry_pos = dir_pos + 16L
+                entry_pos = dir_pos + 16
 
         tmp_avail = current_dir[0]
         tmp_used = current_dir[3]
 
         #Fill directory with empty data until directory is full
         while current_dir[0] > 0:
-            entry_pos = dir_pos + 16L + (16L * current_dir[3])
-            self._write_data(np.array([0,0,0,0]), imgf, entry_pos, np.uint32)
+            entry_pos = dir_pos + 16 + (16 * current_dir[3])
+            self._write_data(np.zeros(4, dtype=np.int32), imgf, entry_pos)
             current_dir[0] = current_dir[0] - 1
             current_dir[3] = current_dir[3] + 1
 
