@@ -1,11 +1,3 @@
-# emacs: -*- mode: python-mode; py-indent-offset: 4; indent-tabs-mode: nil -*-
-# vi: set ft=python sts=4 ts=4 sw=4 et:
-### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-#
-#   See COPYING file distributed along with the NiBabel package for the
-#   copyright and license terms.
-#
-### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 """
 NetCDF reader/writer module.
 
@@ -14,142 +6,27 @@ accessed through the `netcdf_file` object. Data written to and from NetCDF
 files are contained in `netcdf_variable` objects. Attributes are given
 as member variables of the `netcdf_file` and `netcdf_variable` objects.
 
-Notes
------
-NetCDF files are a self-describing binary data format. The file contains
-metadata that describes the dimensions and variables in the file. More
-details about NetCDF files can be found `here
-<http://www.unidata.ucar.edu/software/netcdf/docs/netcdf.html>`_. There
-are three main sections to a NetCDF data structure:
-
-1. Dimensions
-2. Variables
-3. Attributes
-
-The dimensions section records the name and length of each dimension used
-by the variables. The variables would then indicate which dimensions it
-uses and any attributes such as data units, along with containing the data
-values for the variable. It is good practice to include a
-variable that is the same name as a dimension to provide the values for
-that axes. Lastly, the attributes section would contain additional
-information such as the name of the file creator or the instrument used to
-collect the data.
-
-When writing data to a NetCDF file, there is often the need to indicate the
-'record dimension'. A record dimension is the unbounded dimension for a
-variable. For example, a temperature variable may have dimensions of
-latitude, longitude and time. If one wants to add more temperature data to
-the NetCDF file as time progresses, then the temperature variable should
-have the time dimension flagged as the record dimension.
-
 This module implements the Scientific.IO.NetCDF API to read and create
 NetCDF files. The same API is also used in the PyNIO and pynetcdf
 modules, allowing these modules to be used interchangeably when working
-with NetCDF files. The major advantage of this module over other
-modules is that it doesn't require the code to be linked to the NetCDF
-C libraries.
-
-The code is based on the `NetCDF file format specification
-<http://www.unidata.ucar.edu/software/netcdf/docs/netcdf.html>`_. A
-NetCDF file is a self-describing binary format, with a header followed
-by data. The header contains metadata describing dimensions, variables
-and the position of the data in the file, so access can be done in an
-efficient manner without loading unnecessary data into memory. We use
-the ``mmap`` module to create Numpy arrays mapped to the data on disk,
-for the same purpose.
-
-The structure of a NetCDF file is as follows:
-
-    C D F <VERSION BYTE> <NUMBER OF RECORDS>
-    <DIMENSIONS> <GLOBAL ATTRIBUTES> <VARIABLES METADATA>
-    <NON-RECORD DATA> <RECORD DATA>
-
-Record data refers to data where the first axis can be expanded at
-will. All record variables share a same dimension at the first axis,
-and they are stored at the end of the file per record, ie
-
-    A[0], B[0], ..., A[1], B[1], ..., etc,
-    
-so that new data can be appended to the file without changing its original
-structure. Non-record data are padded to a 4n bytes boundary. Record data
-are also padded, unless there is exactly one record variable in the file,
-in which case the padding is dropped.  All data is stored in big endian
-byte order.
-
-The Scientific.IO.NetCDF API allows attributes to be added directly to
-instances of ``netcdf_file`` and ``netcdf_variable``. To differentiate
-between user-set attributes and instance attributes, user-set attributes
-are automatically stored in the ``_attributes`` attribute by overloading
-``__setattr__``. This is the reason why the code sometimes uses
-``obj.__dict__['key'] = value``, instead of simply ``obj.key = value``;
-otherwise the key would be inserted into userspace attributes.
-
-In addition, the NetCDF file header contains the position of the data in
-the file, so access can be done in an efficient manner without loading
-unnecessary data into memory. It uses the ``mmap`` module to create
-Numpy arrays mapped to the data on disk, for the same purpose.
-
-Examples
---------
-To create a NetCDF file:
-
-Make a temporary file for testing:
-
-    >>> import os
-    >>> from tempfile import mkdtemp
-    >>> tmp_pth = mkdtemp()
-    >>> fname = os.path.join(tmp_pth, 'test.nc')
-
-Then:
-
-    >>> f = netcdf_file(fname, 'w')
-    >>> f.history = 'Created for a test'
-    >>> f.createDimension('time', 10)
-    >>> time = f.createVariable('time', 'i', ('time',))
-    >>> time[:] = range(10)
-    >>> time.units = 'days since 2008-01-01'
-    >>> f.close()
-
-Note the assignment of ``range(10)`` to ``time[:]``.  Exposing the slice
-of the time variable allows for the data to be set in the object, rather
-than letting ``range(10)`` overwrite the ``time`` variable.
-
-To read the NetCDF file we just created:
-
-    >>> f = netcdf_file(fname, 'r')
-    >>> f.history  #23dt next : bytes
-    'Created for a test'
-    >>> time = f.variables['time']
-    >>> time.units  #23dt next : bytes
-    'days since 2008-01-01'
-    >>> time.shape == (10,)
-    True
-    >>> print time[-1]
-    9
-    >>> f.close()
-
- Delete our temporary directory and file:
-
-    >>> del f, time # needed for windows unlink
-    >>> os.unlink(fname)
-    >>> os.rmdir(tmp_pth)
+with NetCDF files.
 """
 
+from __future__ import division, print_function, absolute_import
 
-
-#TODO:
+# TODO:
 # * properly implement ``_FillValue``.
 # * implement Jeff Whitaker's patch for masked variables.
 # * fix character variables.
 # * implement PAGESIZE for Python 2.6?
 
-#The Scientific.IO.NetCDF API allows attributes to be added directly to
-#instances of ``netcdf_file`` and ``netcdf_variable``. To differentiate
-#between user-set attributes and instance attributes, user-set attributes
-#are automatically stored in the ``_attributes`` attribute by overloading
+# The Scientific.IO.NetCDF API allows attributes to be added directly to
+# instances of ``netcdf_file`` and ``netcdf_variable``. To differentiate
+# between user-set attributes and instance attributes, user-set attributes
+# are automatically stored in the ``_attributes`` attribute by overloading
 #``__setattr__``. This is the reason why the code sometimes uses
 #``obj.__dict__['key'] = value``, instead of simply ``obj.key = value``;
-#otherwise the key would be inserted into userspace attributes.
+# otherwise the key would be inserted into userspace attributes.
 
 
 __all__ = ['netcdf_file']
@@ -162,29 +39,32 @@ import numpy as np
 from ..py3k import asbytes, asstr
 from numpy import fromstring, ndarray, dtype, empty, array, asarray
 from numpy import little_endian as LITTLE_ENDIAN
+from functools import reduce
+
+from .six import integer_types
 
 
-ABSENT       = asbytes('\x00\x00\x00\x00\x00\x00\x00\x00')
-ZERO         = asbytes('\x00\x00\x00\x00')
-NC_BYTE      = asbytes('\x00\x00\x00\x01')
-NC_CHAR      = asbytes('\x00\x00\x00\x02')
-NC_SHORT     = asbytes('\x00\x00\x00\x03')
-NC_INT       = asbytes('\x00\x00\x00\x04')
-NC_FLOAT     = asbytes('\x00\x00\x00\x05')
-NC_DOUBLE    = asbytes('\x00\x00\x00\x06')
-NC_DIMENSION = asbytes('\x00\x00\x00\n')
-NC_VARIABLE  = asbytes('\x00\x00\x00\x0b')
-NC_ATTRIBUTE = asbytes('\x00\x00\x00\x0c')
+ABSENT = b'\x00\x00\x00\x00\x00\x00\x00\x00'
+ZERO = b'\x00\x00\x00\x00'
+NC_BYTE = b'\x00\x00\x00\x01'
+NC_CHAR = b'\x00\x00\x00\x02'
+NC_SHORT = b'\x00\x00\x00\x03'
+NC_INT = b'\x00\x00\x00\x04'
+NC_FLOAT = b'\x00\x00\x00\x05'
+NC_DOUBLE = b'\x00\x00\x00\x06'
+NC_DIMENSION = b'\x00\x00\x00\n'
+NC_VARIABLE = b'\x00\x00\x00\x0b'
+NC_ATTRIBUTE = b'\x00\x00\x00\x0c'
 
 
-TYPEMAP = { NC_BYTE:   ('b', 1),
-            NC_CHAR:   ('c', 1),
-            NC_SHORT:  ('h', 2),
-            NC_INT:    ('i', 4),
-            NC_FLOAT:  ('f', 4),
-            NC_DOUBLE: ('d', 8) }
+TYPEMAP = {NC_BYTE: ('b', 1),
+            NC_CHAR: ('c', 1),
+            NC_SHORT: ('h', 2),
+            NC_INT: ('i', 4),
+            NC_FLOAT: ('f', 4),
+            NC_DOUBLE: ('d', 8)}
 
-REVERSE = { ('b', 1): NC_BYTE,
+REVERSE = {('b', 1): NC_BYTE,
             ('B', 1): NC_CHAR,
             ('c', 1): NC_CHAR,
             ('h', 2): NC_SHORT,
@@ -195,7 +75,7 @@ REVERSE = { ('b', 1): NC_BYTE,
             # these come from asarray(1).dtype.char and asarray('foo').dtype.char,
             # used when getting the types from generic attributes.
             ('l', 4): NC_INT,
-            ('S', 1): NC_CHAR }
+            ('S', 1): NC_CHAR}
 
 
 class netcdf_file(object):
@@ -225,40 +105,111 @@ class netcdf_file(object):
     version : {1, 2}, optional
         version of netcdf to read / write, where 1 means *Classic
         format* and 2 means *64-bit offset format*.  Default is 1.  See
-        `here <http://www.unidata.ucar.edu/software/netcdf/docs/netcdf/Which-Format.html>`_
+        `here <https://www.unidata.ucar.edu/software/netcdf/docs/netcdf/Which-Format.html>`_
         for more info.
 
+    Notes
+    -----
+    The major advantage of this module over other modules is that it doesn't
+    require the code to be linked to the NetCDF libraries. This module is
+    derived from `pupynere <https://bitbucket.org/robertodealmeida/pupynere/>`_.
+
+    NetCDF files are a self-describing binary data format. The file contains
+    metadata that describes the dimensions and variables in the file. More
+    details about NetCDF files can be found `here
+    <https://www.unidata.ucar.edu/software/netcdf/docs/netcdf.html>`_. There
+    are three main sections to a NetCDF data structure:
+
+    1. Dimensions
+    2. Variables
+    3. Attributes
+
+    The dimensions section records the name and length of each dimension used
+    by the variables. The variables would then indicate which dimensions it
+    uses and any attributes such as data units, along with containing the data
+    values for the variable. It is good practice to include a
+    variable that is the same name as a dimension to provide the values for
+    that axes. Lastly, the attributes section would contain additional
+    information such as the name of the file creator or the instrument used to
+    collect the data.
+
+    When writing data to a NetCDF file, there is often the need to indicate the
+    'record dimension'. A record dimension is the unbounded dimension for a
+    variable. For example, a temperature variable may have dimensions of
+    latitude, longitude and time. If one wants to add more temperature data to
+    the NetCDF file as time progresses, then the temperature variable should
+    have the time dimension flagged as the record dimension.
+
+    In addition, the NetCDF file header contains the position of the data in
+    the file, so access can be done in an efficient manner without loading
+    unnecessary data into memory. It uses the ``mmap`` module to create
+    Numpy arrays mapped to the data on disk, for the same purpose.
+
+    Examples
+    --------
+    To create a NetCDF file:
+
+    Make a temporary file for testing:
+
+        >>> import os
+        >>> from tempfile import mkdtemp
+        >>> tmp_pth = mkdtemp()
+        >>> fname = os.path.join(tmp_pth, 'test.nc')
+
+    Then:
+
+        >>> f = netcdf_file(fname, 'w')
+        >>> f.history = 'Created for a test'
+        >>> f.createDimension('time', 10)
+        >>> time = f.createVariable('time', 'i', ('time',))
+        >>> time[:] = np.arange(10)
+        >>> time.units = 'days since 2008-01-01'
+        >>> f.close()
+
+    Note the assignment of ``range(10)`` to ``time[:]``.  Exposing the slice
+    of the time variable allows for the data to be set in the object, rather
+    than letting ``range(10)`` overwrite the ``time`` variable.
+
+    To read the NetCDF file we just created:
+
+        >>> f = netcdf_file(fname, 'r')
+        >>> f.history == b'Created for a test'
+        True
+        >>> time = f.variables['time']
+        >>> time.units == b'days since 2008-01-01'
+        True
+        >>> time.shape == (10,)
+        True
+        >>> time[-1]
+        9
+        >>> f.close()
+
+    A NetCDF file can also be used as context manager:
+
+        >>> with netcdf_file(fname, 'r') as f:
+        ...     print(f.variables['time'].shape == (10,))
+        True
+
+    Delete our temporary directory and file:
+
+        >>> del f, time # needed for windows unlink
+        >>> os.unlink(fname)
+        >>> os.rmdir(tmp_pth)
     """
     def __init__(self, filename, mode='r', mmap=None, version=1):
-        """Initialize netcdf_file from fileobj (str or file-like).
-
-        Parameters
-        ----------
-        filename : string or file-like
-           string -> filename
-        mode : {'r', 'w'}, optional
-           read-write mode, default is 'r'
-        mmap : None or bool, optional
-           Whether to mmap `filename` when reading.  Default is True
-           when `filename` is a file name, False when `filename` is a
-           file-like object
-        version : {1, 2}, optional
-           version of netcdf to read / write, where 1 means *Classic
-           format* and 2 means *64-bit offset format*.  Default is 1.  See
-           http://www.unidata.ucar.edu/software/netcdf/docs/netcdf/Which-Format.html#Which-Format
-        """
-        if hasattr(filename, 'seek'): # file-like
+        """Initialize netcdf_file from fileobj (str or file-like)."""
+        if hasattr(filename, 'seek'):  # file-like
             self.fp = filename
             self.filename = 'None'
             if mmap is None:
                 mmap = False
             elif mmap and not hasattr(filename, 'fileno'):
                 raise ValueError('Cannot use file object for mmap')
-        else: # maybe it's a string
+        else:  # maybe it's a string
             self.filename = filename
             self.fp = open(self.filename, '%sb' % mode)
             if mmap is None:
-                mmap  = True
+                mmap = True
         self.use_mmap = mmap
         self.version_byte = version
 
@@ -289,17 +240,19 @@ class netcdf_file(object):
 
     def close(self):
         """Closes the NetCDF file."""
-        try:
-            if self.fp.closed:
-                return
-        except AttributeError: # gzip files don't have closed attr
-            pass
-        try:
-            self.flush()
-        finally:
-            self.fp.close()
+        if not self.fp.closed:
+            try:
+                self.flush()
+            finally:
+                self.fp.close()
     __del__ = close
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
     def createDimension(self, name, length):
         """
         Adds a dimension to the Dimension section of the NetCDF data structure.
@@ -357,14 +310,12 @@ class netcdf_file(object):
         shape = tuple([self.dimensions[dim] for dim in dimensions])
         shape_ = tuple([dim or 0 for dim in shape])  # replace None with 0 for numpy
 
-        if isinstance(type, basestring): type = dtype(type)
+        type = dtype(type)
         typecode, size = type.char, type.itemsize
         if (typecode, size) not in REVERSE:
             raise ValueError("NetCDF 3 does not support type %s" % type)
-        dtype_ = '>%s' % typecode
-        if size > 1: dtype_ += str(size)
 
-        data = empty(shape_, dtype=dtype_)
+        data = empty(shape_, dtype=type.newbyteorder("B"))  # convert to big endian always for NetCDF 3
         self.variables[name] = netcdf_variable(data, typecode, size, shape, dimensions)
         return self.variables[name]
 
@@ -382,7 +333,8 @@ class netcdf_file(object):
     sync = flush
 
     def _write(self):
-        self.fp.write(asbytes('CDF'))
+        self.fp.seek(0)
+        self.fp.write(b'CDF')
         self.fp.write(array(self.version_byte, '>b').tostring())
 
         # Write headers and data.
@@ -429,9 +381,9 @@ class netcdf_file(object):
 
             # Sort variables non-recs first, then recs. We use a DSU
             # since some people use pupynere with Python 2.3.x.
-            deco = [ (v._shape and not v.isrec, k) for (k, v) in self.variables.items() ]
+            deco = [(v._shape and not v.isrec, k) for (k, v) in self.variables.items()]
             deco.sort()
-            variables = [ k for (unused, k) in deco ][::-1]
+            variables = [k for (unused, k) in deco][::-1]
 
             # Set the metadata for all variables.
             for name in variables:
@@ -493,7 +445,7 @@ class netcdf_file(object):
         if not var.isrec:
             self.fp.write(var.data.tostring())
             count = var.data.size * var.data.itemsize
-            self.fp.write(asbytes('0') * (var._vsize - count))
+            self.fp.write(b'0' * (var._vsize - count))
         else:  # record variable
             # Handle rec vars with shape[0] < nrecs.
             if self._recs > len(var.data):
@@ -511,7 +463,7 @@ class netcdf_file(object):
                 self.fp.write(rec.tostring())
                 # Padding
                 count = rec.size * rec.itemsize
-                self.fp.write(asbytes('0') * (var._vsize - count))
+                self.fp.write(b'0' * (var._vsize - count))
                 pos += self._recsize
                 self.fp.seek(pos)
             self.fp.seek(pos0 + var._vsize)
@@ -520,18 +472,18 @@ class netcdf_file(object):
         if hasattr(values, 'dtype'):
             nc_type = REVERSE[values.dtype.char, values.dtype.itemsize]
         else:
-            types = [
-                    (int, NC_INT),
-                    (long, NC_INT),
+            types = [(t, NC_INT) for t in integer_types]
+            types += [
                     (float, NC_FLOAT),
-                    (basestring, NC_CHAR),
+                    (str, NC_CHAR),
                     ]
             try:
                 sample = values[0]
             except TypeError:
                 sample = values
             for class_, nc_type in types:
-                if isinstance(sample, class_): break
+                if isinstance(sample, class_):
+                    break
 
         typecode, size = TYPEMAP[nc_type]
         dtype_ = '>%s' % typecode
@@ -551,12 +503,12 @@ class netcdf_file(object):
             values = values.byteswap()
         self.fp.write(values.tostring())
         count = values.size * values.itemsize
-        self.fp.write(asbytes('0') * (-count % 4))  # pad
+        self.fp.write(b'0' * (-count % 4))  # pad
 
     def _read(self):
         # Check magic bytes and version
         magic = self.fp.read(3)
-        if not magic == asbytes('CDF'):
+        if not magic == b'CDF':
             raise TypeError("Error: %s is not a valid NetCDF 3 file" %
                             self.filename)
         self.__dict__['version_byte'] = fromstring(self.fp.read(1), '>b')[0]
@@ -610,7 +562,7 @@ class netcdf_file(object):
         for var in range(count):
             (name, dimensions, shape, attributes,
              typecode, size, dtype_, begin_, vsize) = self._read_var()
-            # http://www.unidata.ucar.edu/software/netcdf/docs/netcdf.html
+            # https://www.unidata.ucar.edu/software/netcdf/docs/netcdf.html
             # Note that vsize is the product of the dimension lengths
             # (omitting the record dimension) and the number of bytes
             # per value (determined from the type), increased to the
@@ -624,12 +576,13 @@ class netcdf_file(object):
             # 32-bit vsize field is not large enough to contain the size
             # of variables that require more than 2^32 - 4 bytes, so
             # 2^32 - 1 is used in the vsize field for such variables.
-            if shape and shape[0] is None: # record variable
+            if shape and shape[0] is None:  # record variable
                 rec_vars.append(name)
                 # The netCDF "record size" is calculated as the sum of
                 # the vsize's of all the record variables.
                 self.__dict__['_recsize'] += vsize
-                if begin == 0: begin = begin_
+                if begin == 0:
+                    begin = begin_
                 dtypes['names'].append(name)
                 dtypes['formats'].append(str(shape[1:]) + dtype_)
 
@@ -643,7 +596,7 @@ class netcdf_file(object):
 
                 # Data will be set later.
                 data = None
-            else: # not a record variable
+            else:  # not a record variable
                 # Calculate size to avoid problems with vsize (above)
                 a_size = reduce(mul, shape, 1) * size
                 if self.use_mmap:
@@ -719,9 +672,10 @@ class netcdf_file(object):
 
         if typecode is not 'c':
             values = fromstring(values, dtype='>%s' % typecode)
-            if values.shape == (1,): values = values[0]
+            if values.shape == (1,):
+                values = values[0]
         else:
-            values = values.rstrip(asbytes('\x00'))
+            values = values.rstrip(b'\x00')
         return values
 
     def _pack_begin(self, begin):
@@ -748,11 +702,11 @@ class netcdf_file(object):
         count = len(s)
         self._pack_int(count)
         self.fp.write(asbytes(s))
-        self.fp.write(asbytes('0') * (-count % 4))  # pad
+        self.fp.write(b'0' * (-count % 4))  # pad
 
     def _unpack_string(self):
         count = self._unpack_int()
-        s = self.fp.read(count).rstrip(asbytes('\x00'))
+        s = self.fp.read(count).rstrip(b'\x00')
         self.fp.read(-count % 4)  # read padding
         return s
 
@@ -878,6 +832,14 @@ class netcdf_variable(object):
             netcdf variable.
 
         """
+        if not self.data.flags.writeable:
+            # Work-around for a bug in NumPy.  Calling itemset() on a read-only
+            # memory-mapped array causes a seg. fault.
+            # See NumPy ticket #1622, and SciPy ticket #1202.
+            # This check for `writeable` can be removed when the oldest version
+            # of numpy still supported by scipy contains the fix for #1622.
+            raise RuntimeError("variable is not writeable")
+
         self.data.itemset(value)
 
     def typecode(self):
