@@ -1,7 +1,7 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 ''' Tests for data module '''
-from __future__ import with_statement
+from __future__ import division, print_function, absolute_import
 import os
 from os.path import join as pjoin
 from os import environ as env
