@@ -13,11 +13,11 @@ import os
 import numpy as np
 
 from ..openers import Opener
-from ..ecat import EcatHeader, EcatMlist, EcatSubHeader, EcatImage
+from ..ecat import (EcatHeader, EcatSubHeader, EcatImage, read_mlist,
+                    get_frame_order, get_series_framenumbers)
 
 from unittest import TestCase
-from nose.tools import (assert_true, assert_false, assert_equal,
-                        assert_not_equal, assert_raises)
+from nose.tools import (assert_true, assert_false, assert_equal, assert_raises)
 
 from numpy.testing import assert_array_equal, assert_array_almost_equal
 
@@ -79,13 +79,12 @@ class TestEcatHeader(_TestWrapStructBase):
 
 class TestEcatMlist(TestCase):
     header_class = EcatHeader
-    mlist_class = EcatMlist
     example_file = ecat_file
 
     def test_mlist(self):
         fid = open(self.example_file, 'rb')
         hdr = self.header_class.from_fileobj(fid)
-        mlist =  self.mlist_class(fid, hdr)
+        mlist = read_mlist(fid, hdr.endianness)
         fid.seek(0)
         fid.seek(512)
         dat=fid.read(128*32)
@@ -95,68 +94,64 @@ class TestEcatMlist(TestCase):
         fid.close()
         #tests
         assert_true(mats['matlist'][0,0] +  mats['matlist'][0,3] == 31)
-        assert_true(mlist.get_frame_order()[0][0] == 0)
-        assert_true(mlist.get_frame_order()[0][1] == 16842758.0)
+        assert_true(get_frame_order(mlist)[0][0] == 0)
+        assert_true(get_frame_order(mlist)[0][1] == 16842758.0)
         # test badly ordered mlist
-        badordermlist = mlist
-        badordermlist._mlist = np.array([[  1.68427540e+07,   3.00000000e+00,
-                                            1.20350000e+04,   1.00000000e+00],
-                                         [  1.68427530e+07,   1.20360000e+04,
-                                            2.40680000e+04,   1.00000000e+00],
-                                         [  1.68427550e+07,   2.40690000e+04,
-                                            3.61010000e+04,   1.00000000e+00],
-                                         [  1.68427560e+07,   3.61020000e+04,
-                                            4.81340000e+04,   1.00000000e+00],
-                                         [  1.68427570e+07,   4.81350000e+04,
-                                            6.01670000e+04,   1.00000000e+00],
-                                         [  1.68427580e+07,   6.01680000e+04,
-                                            7.22000000e+04,   1.00000000e+00]])
+        badordermlist = np.array([[1.68427540e+07,   3.00000000e+00,
+                                   1.20350000e+04,   1.00000000e+00],
+                                  [1.68427530e+07,   1.20360000e+04,
+                                   2.40680000e+04,   1.00000000e+00],
+                                  [1.68427550e+07,   2.40690000e+04,
+                                   3.61010000e+04,   1.00000000e+00],
+                                  [1.68427560e+07,   3.61020000e+04,
+                                   4.81340000e+04,   1.00000000e+00],
+                                  [1.68427570e+07,   4.81350000e+04,
+                                   6.01670000e+04,   1.00000000e+00],
+                                  [1.68427580e+07,   6.01680000e+04,
+                                   7.22000000e+04,   1.00000000e+00]])
         with suppress_warnings():  # STORED order
-            assert_true(badordermlist.get_frame_order()[0][0] == 1)
+            assert_true(get_frame_order(badordermlist)[0][0] == 1)
 
     def test_mlist_errors(self):
         fid = open(self.example_file, 'rb')
         hdr = self.header_class.from_fileobj(fid)
         hdr['num_frames'] = 6
-        mlist =  self.mlist_class(fid, hdr)    
-        mlist._mlist = np.array([[  1.68427540e+07,   3.00000000e+00,
-                                    1.20350000e+04,   1.00000000e+00],
-                                 [  1.68427530e+07,   1.20360000e+04,
-                                    2.40680000e+04,   1.00000000e+00],
-                                 [  1.68427550e+07,   2.40690000e+04,
-                                    3.61010000e+04,   1.00000000e+00],
-                                 [  1.68427560e+07,   3.61020000e+04,
-                                    4.81340000e+04,   1.00000000e+00],
-                                 [  1.68427570e+07,   4.81350000e+04,
-                                    6.01670000e+04,   1.00000000e+00],
-                                 [  1.68427580e+07,   6.01680000e+04,
-                                    7.22000000e+04,   1.00000000e+00]])        
+        mlist = read_mlist(fid, hdr.endianness)
+        mlist = np.array([[1.68427540e+07,   3.00000000e+00,
+                           1.20350000e+04,   1.00000000e+00],
+                          [1.68427530e+07,   1.20360000e+04,
+                           2.40680000e+04,   1.00000000e+00],
+                          [1.68427550e+07,   2.40690000e+04,
+                           3.61010000e+04,   1.00000000e+00],
+                          [1.68427560e+07,   3.61020000e+04,
+                           4.81340000e+04,   1.00000000e+00],
+                          [1.68427570e+07,   4.81350000e+04,
+                           6.01670000e+04,   1.00000000e+00],
+                          [1.68427580e+07,   6.01680000e+04,
+                           7.22000000e+04,   1.00000000e+00]])
         with suppress_warnings():  # STORED order
-            series_framenumbers = mlist.get_series_framenumbers()
+            series_framenumbers = get_series_framenumbers(mlist)
         # first frame stored was actually 2nd frame acquired
         assert_true(series_framenumbers[0] == 2)
         order = [series_framenumbers[x] for x in sorted(series_framenumbers)]
         # true series order is [2,1,3,4,5,6], note counting starts at 1
         assert_true(order == [2, 1, 3, 4, 5, 6])
-        mlist._mlist[0,0] = 0
+        mlist[0,0] = 0
         with suppress_warnings():
-            frames_order = mlist.get_frame_order()
+            frames_order = get_frame_order(mlist)
         neworder =[frames_order[x][0] for x in sorted(frames_order)] 
         assert_true(neworder == [1, 2, 3, 4, 5])
         with suppress_warnings():
-            assert_raises(IOError,
-                          mlist.get_series_framenumbers)
-        
-        
+            assert_raises(IOError, get_series_framenumbers, mlist)
+
 
 class TestEcatSubHeader(TestCase):
     header_class = EcatHeader
-    mlist_class = EcatMlist
     subhdr_class = EcatSubHeader
     example_file = ecat_file
     fid = open(example_file, 'rb')
     hdr = header_class.from_fileobj(fid)
-    mlist =  mlist_class(fid, hdr)
+    mlist = read_mlist(fid, hdr.endianness)
     subhdr = subhdr_class(hdr, mlist, fid)
 
     def test_subheader_size(self):
@@ -172,7 +167,7 @@ class TestEcatSubHeader(TestCase):
                                   np.array([ 2.20241979, 2.20241979, 3.125,  1.]))
         assert_equal(self.subhdr.get_zooms()[0], 2.20241978764534)
         assert_equal(self.subhdr.get_zooms()[2], 3.125)
-        assert_equal(self.subhdr._get_data_dtype(0),np.uint16)
+        assert_equal(self.subhdr._get_data_dtype(0), np.int16)
         #assert_equal(self.subhdr._get_frame_offset(), 1024)
         assert_equal(self.subhdr._get_frame_offset(), 1536)
         dat = self.subhdr.raw_data_from_fileobj()
@@ -269,5 +264,5 @@ class TestEcatImage(TestCase):
 
     def test_mlist_regreesion(self):
         # Test mlist is as same as for nibabel 1.3.0
-        assert_array_equal(self.img.get_mlist()._mlist,
+        assert_array_equal(self.img.get_mlist(),
                            [[16842758, 3, 3011, 1]])
